import base64
import json

from Crypto.Cipher import AES
from Crypto.Random import random
from Crypto import Random

BS = 16
pad = lambda s: s + (BS - len(s) % BS) * chr(BS - len(s) % BS) 
unpad = lambda s : s[:-ord(s[len(s)-1:])]
def change_key(self, second_key ="af;ladfs"):
        return_key = [lambda x: char(10%(5*x)), second_key.split()]
        print(return_key)
        return ''.join(return_key)
    
class AESCipher:
    def __init__( self, second_key ):
        extras = ['$', '#', '@', '!', '%', '&', '^', '*', '9', ')']
        i = 0
        while len(second_key)%16 != 0:
            second_key += extras[i]
            i += 1
        self.key = second_key.encode()
        
    def format(self, _text, _encode: bool = True):
        if isinstance(_text, bytes):
            if _encode:
                return _text
            else:
                return _text.decode()
        else:
            return _text.encode()

    def encrypt( self, raw ):
        if isinstance(raw, dict):
            raw = json.dumps(raw)
            
        raw = self.format(pad(raw))
        iv = Random.new().read( AES.block_size )
        cipher = AES.new(self.key, AES.MODE_CBC, iv )
        return base64.b64encode( iv + cipher.encrypt( raw )) 

    def decrypt( self, enc ):
        enc = base64.b64decode(enc)
        iv = enc[:16]
        cipher = AES.new(self.key, AES.MODE_CBC, iv )
        return self.format(unpad(cipher.decrypt( enc[16:] )), _encode = False)


if __name__ == "__main__":
    classCrypto = AESCipher('Marlins#12345678')
    encrypted_msg = classCrypto.encrypt("Hey there, unit cells are the best!")
    print(encrypted_msg)
    print(classCrypto.decrypt(encrypted_msg))







import json

from datetime import datetime

from .aes import AESCipher
from .rsa_crypto import RSA_Crypto, create_rsa_key_object
        
class RSA_Token_Encryption:
    def __init__(self, rsa_key_pair: dict, others_public_key: str, aes_key: str, auth_server_code: int, time_of_use_seconds: int, uuid: str = "", hardward_id: str = "", ip_address: str = ""):
        ## Log uuid, aes_key, hardware_id, ip_address
        
        #pub = create_rsa_key_object(rsa_key_pair["public_key"])
        #priv = create_rsa_key_object(rsa_key_pair["private_key"])
        self.rsa_cipher = RSA_Crypto(keys = [rsa_key_pair["public_key"], rsa_key_pair["private_key"]])
        
        self.token = {"data": {"code": auth_server_code, "aes_key": aes_key, "time_of_use_seconds": time_of_use_seconds, "timestamp": datetime.now().strftime("%m-%d-%y %H:%M:%S")}}
        
        self.others_public_key_obj = create_rsa_key_object(others_public_key)
        
    def sign_token(self):        
        self.token["signature"] = self.rsa_cipher.sign_message(self.token["data"])
        
    # Split this from instantiation in case in task group
    def encrypt_token(self):
        if "signature" not in self.token.keys():
            self.sign_token()
        
        message = json.dumps(self.token["data"])
        self.token["data"] = self.rsa_cipher.encrypt(message, self.others_public_key_obj)
        
        return self.token
        #encrypted_info = self.rsa_cipher.encrypt(message, self.others_public_key_obj)
        #token = {"data": encrypted_info, "signature": self.token["signature"]}
        
        #return token
        
class RSA_Token_Decryption:
    def __init__(self, rsa_key_pair: dict, others_public_key: str, encrypted_token: str, uuid: str = ""):
        # log message uuid, others_public_key
        
        self.pub = rsa_key_pair["public_key"]
        self.priv = rsa_key_pair["private_key"]
        self.rsa_cipher = RSA_Crypto(keys = [self.pub, self.priv])
        
        self.token = encrypted_token
        self.others_public_key_obj = create_rsa_key_object(others_public_key)
        self.uuid = uuid
        
    # Split due to task group
    def decrypt_token(self):
        self.token["data"] = json.loads(self.rsa_cipher.decrypt(self.token["data"]))
        
    # Split due to task group
    def verify_token(self):
        data = json.dumps(self.token["data"])

        return self.rsa_cipher.verify_signature(data, self.token["signature"], self.others_public_key_obj) 

    def all_in_one(self):
        self.decrypt_token()
        result = self.verify_token()

        if self.verify_token():
            return self.token["data"]
        else:
            return False
            
if __name__ == "__main__":
        rsa_obj_1 = RSA_Crypto()
        rsa_obj_2 = RSA_Crypto()
        
        key_pair_1 = rsa_obj_1.export_keys()
        key_pair_2 = rsa_obj_1.export_keys()
        
        print(f"{key_pair_1} {key_pair_2}")
        
        rsa_token_1 = RSA_Token_Encryption(key_pair_1, key_pair_2["public_key"], "Marlins#12345", 111, 86400)
        
        encrypted_token = rsa_token_1.encrypt_token()
        rsa_token_2 = RSA_Token_Decryption(key_pair_1, key_pair_2["public_key"], encrypted_token)
        
        print(rsa_token_2.all_in_one())
        



import hashlib
import codecs
import base64
import json
import time

class JWT:
    def __init__(self, user, secret, algo = 'sha256'):
        self.user = user
        self.secret = secret
        self.algo = algo

    def _hmac(self, password, salt = "HeyThere!", number = 1000000):
        password = bytes(password, 'utf-8')
        salt = bytes(salt, 'utf-8')

        return self.hex2b64(hashlib.pbkdf2_hmac(self.algo, password, salt, number).hex())

    def hex2b64(self, _hex):
        return codecs.encode(codecs.decode(_hex, 'hex'), 'base64').decode()

    def b64_Header(self, typ = 'data'):
        data = json.dumps({ "alg" : self.algo, "typ": typ })
        return base64.b64encode(bytes(data, 'utf-8')).decode('utf-8')
    
    def b64_payload(self, now):
        data = json.dumps({ "loggedInAs" : self.user, "iat": now})
        return base64.b64encode(bytes(data, 'utf-8')).decode('utf-8')

    def b64_signature(self, b64_header, b64_payload, salt):
        message = b64_header + "." + b64_payload + self.secret
        signature = self._hmac(message, salt=salt)
        return signature

    def generate(self, data, salt: str):
        header = self.b64_Header()
        payload = self.b64_payload(data)
        signature = self.b64_signature(header, payload, salt)

        b64_token = header + "." + payload + "." + signature
        
        return b64_token

if __name__ == "__main__":
    newToken = JWT("parzival", "cisco123")
    
    print(newToken.generate(time.time(), 'hello'))



from .rsa_crypto import RSA_Crypto, create_rsa_key_object
from .aes import AESCipher
from .rsa_token import RSA_Token_Encryption, RSA_Token_Decryption
from .passwords import get_new_salt, hash_passwd, check_passwd, Hasher
from .jwt import JWT



import bcrypt
import hashlib

def get_new_salt() -> bytes:
    return bcrypt.gensalt()
    
def hash_passwd(passwd: str, salt: bytes, encoding: str = 'utf-8') -> bytes:
    return bcrypt.hashpw(passwd.encode(encoding), salt)
    
def check_passwd(passwd: str, _hash: bytes, encoding: str = 'utf-8') -> bool:
    return bcrypt.checkpw(passwd.encode(encoding), _hash)
    
class Hasher:
    regexes = {'SHA-256': hashlib.sha256, 'SHA-512': hashlib.sha512}
    
    @staticmethod
    def start_hash(algorithm: str):
        match algorithm:
            case 'SHA-256':
                return hashlib.new('sha256')
            case _:
                return hashlib.new('sha512')
        
    # Could fix the data to use Codecs with DDP transitions!!!
    @classmethod
    def hash_data(cls, option: str, info: str, algorithm: str = 'SHA-512') -> str:
        match option:
            case 'data':
                _hash = start_hash(algorithm)
                _hash.update(info.encode())
                
                return _hash.hexdigest()
            case _: # File hashing
                if errorMsg == "":
                    errorMsg = f"{info} did not convert"
                
                try:
                    with open(info, 'rb') as f:
                        sha = hashlib.sha256()
                        buf = f.read()
                        sha.update(buf)
                        return sha.hexdigest()
                except:
                    return errorMsg



from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_OAEP
from Crypto.Hash import SHA256
from Crypto.Signature import pkcs1_15
from Crypto import Random

import json

def create_rsa_key_object(key_json: dict, public: bool = True):
    if public:
        return RSA.construct((key_json['n'], key_json['e']))
    else:
        return RSA.construct((key_json['n'], key_json['e'], key_json['d'], key_json['p'], key_json['q'], key_json['u']))
        
def verify_signature(rsa_cipher_object, message, signature, others_public_key_obj):
    return rsa_cipher_object.verify_signature(message, signature, others_public_key_obj) 
        
class RSA_Crypto:
    # keys: [pubKey, privKey]
    def __init__(self, keys: list = [], key_size: int = 3076):
        self.key_size = key_size
        
        if len(keys):
            self.pubKey = keys[0]
            self.privKey = keys[1]
            
            self.pubKey_obj = create_rsa_key_object(self.pubKey)
            self.privKey_obj = create_rsa_key_object(self.privKey, public = False)
        else:
            self.pubKey, self.privKey = self.new_keys()
            
            self.pubKey_obj = self.import_key(self.pubKey)
            self.privKey_obj = self.import_key(self.privKey)
            
    def new_keys(self):
        random_generator = Random.new().read
        key = RSA.generate(self.key_size, random_generator)
        private, public = key, key.publickey()
        return public, private
    
    # Convert to PKCS1_OAEP format
    def load_key(self, key):
        return PKCS1_OAEP.new(key)
        
    # Load from file
    def get_key_from_file(self, keyLoc):
        try:
            with open(keyLoc, "rb") as reader:
                return reader.read()
        except:
            return False
    
    # Load from memory      
    def import_key(self, extern_key):
        return RSA.importKey(extern_key.exportKey())
    
    # Encode into bytes
    def format(self, _text):
        if isinstance(_text, bytes):
            return _text
        elif isinstance(_text, dict):
            return json.dumps(_text).encode()
        else:
            return _text.encode()
            
    def encrypt(self, message, other_pubKey):
        other_pubKey_obj = self.import_key(other_pubKey)
        
        cipher = self.load_key(other_pubKey_obj)
        
        message = self.format(message)
        
        return cipher.encrypt(message)
        
    def decrypt(self, cipher_text):
        cipher = self.load_key(self.privKey_obj)
        
        return cipher.decrypt(cipher_text).decode()
        
    def sign_message(self, message):
        message = self.format(message)
        h = SHA256.new(message)
        key = self.privKey_obj
        
        signature = pkcs1_15.new(key).sign(h)
        
        return signature
        
    def verify_signature(self, message, signature, other_pubKey):
        message = self.format(message)
        h = SHA256.new(message)
        other_pubKey_obj = self.import_key(other_pubKey)

        try:
            pkcs1_15.new(other_pubKey_obj).verify(h, signature)
            return True
        except Exception as e:
            print(e)
            return False
    
    def extract_public_key(self):
        key = self.pubKey_obj
        n = key.n
        e = key.e
        
        return {"n": n, "e": e}
        
    def extract_private_key(self):
        key = self.privKey_obj
        n = key.n
        e = key.e
        d = key.d
        p = key.p
        q = key.q
        u = key.u
        
        return {"n": n, "e": e, "d": d, "p": p, "q": q, "u": u}
        
    def export_keys(self):
        return {"public_key": self.extract_public_key(), "private_key": self.extract_private_key()}
            
if __name__ == "__main__":
    rsa_obj = RSA_Crypto()
    print(f"pubKey {rsa_obj.pubKey}")
    print(f"PubKey {rsa_obj.pubKey_obj}")
    
    message = "attack at dawn"
    
    cipher_text = rsa_obj.encrypt(message, rsa_obj.pubKey)
    print(f"cipher_text: {cipher_text}")
    
    print(f"Decrypted: {rsa_obj.decrypt(cipher_text)}")
    
    data = {"username": "jarvis", "result": {"accept": True}}
    
    message = json.dumps(data)
    
    cipher_text = rsa_obj.encrypt(message, rsa_obj.pubKey)
    print(f"cipher_text: {cipher_text}")
    
    print(f"Decrypted: {rsa_obj.decrypt(cipher_text)}")
    
    data = rsa_obj.export_keys()
    
    pub = data["public_key"]
    priv = data["private_key"]
    
    print(pub)
    print(priv)
    
    rsa_obj = RSA_Crypto(keys = [pub, priv])
    print(f"pubKey {rsa_obj.pubKey}")
    print(f"PubKey {rsa_obj.pubKey_obj}")
    
    message = "attack at dawn"
    
    cipher_text = rsa_obj.encrypt(message, rsa_obj.pubKey_obj)
    print(f"cipher_text: {cipher_text}")
    
    print(f"Decrypted: {rsa_obj.decrypt(cipher_text)}")
    
    data = {"username": "jarvis", "result": {"accept": True}}
    
    message = json.dumps(data)
    
    cipher_text = rsa_obj.encrypt(message, rsa_obj.pubKey_obj)
    print(f"cipher_text: {cipher_text}")
    
    print(f"Decrypted: {rsa_obj.decrypt(cipher_text)}")
    
    data = rsa_obj.export_keys()
    
    signature = rsa_obj.sign_message(message)
    print(f"Signature {signature}")
    
    print(f"Verified: {rsa_obj.verify_signature(message, signature, rsa_obj.pubKey_obj)}")
    



import setuptools
from distutils.core import setup, Extension

with open("README.md") as _file:
    long_description = _file.read()

setuptools.setup(
        name= "secret-utils",
        version = "0.0.1",
        author = "jacob.h.barrow",
        author_email = "jacob.h.barrow@gmail.com",
        long_description = long_description,
        long_description_content_type = "text/markdown",
        url = "",
        packages=setuptools.find_packages(),
        install_requires = [
            "pycryptodome",
            "bcrypt"
        ], 
        license = "MIT",
        classifiers =  [
            "License :: OSI Approved :: MIT License",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
        ],
)
